﻿using ProjectPublishing.Entities;
using ProjectPublishing.Repositories;
using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows.Forms;

namespace ProjectPublishing.Forms
{
    public partial class FormCustomer : Form
    {
        private readonly ICustomerRepository _customerRepository;
        private int? _customerId;

        public int Id
        {
            set
            {
                try
                {
                    var customer = _customerRepository.ReadCustomerById(value);
                    if (customer == null)
                    {
                        throw new InvalidDataException(nameof(customer));
                    }
                    textBoxCustomerPhone.Text = customer.PhoneNumber;
                    textBoxCustomerEmail.Text = customer.Email;
                    comboBoxPublisherID.SelectedValue = customer.PublisherId;
                    _customerId = value;
                }
                catch (Exception ex)
                {
                    MessageBox.Show(ex.Message, "Ошибка при получении данных", MessageBoxButtons.OK, MessageBoxIcon.Error);
                    return;
                }
            }
        }

        public FormCustomer(ICustomerRepository customerRepository, IContactPersonRepository contactPersonRepository)
        {
            InitializeComponent();
            _customerRepository = customerRepository ?? throw new ArgumentException(nameof(customerRepository));

            comboBoxPublisherID.DataSource = contactPersonRepository.ReadContactPeople();
            comboBoxPublisherID.DisplayMember = "Name";
            comboBoxPublisherID.ValueMember = "Id";
        }

        private void buttonSave_Click(object sender, EventArgs e)
        {
            try
            {
                if (string.IsNullOrWhiteSpace(textBoxCustomerPhone.Text) || string.IsNullOrWhiteSpace(textBoxCustomerEmail.Text))
                {
                    throw new Exception("Имеются незаполненные поля!");
                }

                if (_customerId.HasValue)
                {
                    _customerRepository.UpdateCustomer(CreateCustomer(_customerId.Value));
                }
                else
                {
                    _customerRepository.CreateCustomer(CreateCustomer(0));
                }
                Close();
            }
            catch (Exception ex)
            {
                MessageBox.Show(ex.Message, "Ошибка при сохранении", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
        }

        private void buttonCancel_Click(object sender, EventArgs e) => Close();

        private Customer CreateCustomer(int id) => Customer.CreateCustomer(id, textBoxCustomerPhone.Text, textBoxCustomerEmail.Text, (int)comboBoxPublisherID.SelectedValue!);
    }
}
