﻿using ProjectPublishing.Entities.Enums;
using ProjectPublishing.Entities;
using ProjectPublishing.Repositories;
using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows.Forms;

namespace ProjectPublishing.Forms
{
    public partial class FormOrder : Form
    {
        private readonly IOrderRepository _orderRepository;

        public FormOrder(IOrderRepository orderRepository, ICustomerRepository customerRepository, IPrintingHouseRepository printingHouseRepository, IMaterialRepository materialRepository)
        {
            InitializeComponent();
            _orderRepository = orderRepository ?? throw new ArgumentException(nameof(orderRepository));
            comboBoxOrderType.DataSource = Enum.GetValues(typeof(ProductType));
            comboBoxStatus.DataSource = Enum.GetValues(typeof(OrderStatus));
            comboBoxCustomerID.DataSource = customerRepository.ReadCustomers();
            comboBoxPrintingUD.DataSource = printingHouseRepository.ReadPrintingHouses();

            comboBoxCustomerID.DisplayMember = "Name";
            comboBoxCustomerID.ValueMember = "Id";

            comboBoxPrintingUD.DisplayMember = "Name";
            comboBoxPrintingUD.ValueMember = "Id";


            columnMaterialType.DataSource = materialRepository.ReadMaterials();
            columnMaterialType.DisplayMember = "Name";
            columnMaterialType.ValueMember= "Id";
        }

        private void buttonSave_Click(object sender, EventArgs e)
        {
            try
            {
                if (dataGridViewMaterials.RowCount < 1)
                {
                    throw new Exception("Имеются незаполненные поля!");
                }

                _orderRepository.CreateOrder(Order.CreateOrder(0, textBoxDesc.Text,
                (ProductType)comboBoxOrderType.SelectedItem!,
                (int)numericUpDownAmount.Value,
                (OrderStatus)comboBoxStatus.SelectedItem!,
                (int)comboBoxCustomerID.SelectedValue!,
                (int)comboBoxPrintingUD.SelectedValue!,
                CreateListFromDataGrid()));
                Close();
            }
            catch (Exception ex)
            {
                MessageBox.Show(ex.Message, "Ошибка при сохранении", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
        }

        private void buttonCancel_Click(object sender, EventArgs e) => Close();

        private List<OrderMaterials> CreateListFromDataGrid()
        {
            var list = new List<OrderMaterials>();
            foreach (DataGridViewRow row in dataGridViewMaterials.Rows)
            {
                if (row.Cells["columnMaterial"].Value == null || row.Cells["columnMaterialType"].Value == null)
                {
                    continue;
                }
                list.Add(OrderMaterials.Create(0, Convert.ToInt32(row.Cells["columnMaterialType"].Value), Convert.ToInt32(row.Cells["columnMaterial"].Value)));
            }
            return list;
        }
    }
}
