﻿using ProjectPublishing.Entities;
using ProjectPublishing.Repositories;
using ProjectPublishing.Repositories.Implementations;
using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows.Forms;

namespace ProjectPublishing.Forms
{
    public partial class FormPrintingHouse : Form
    {
        private readonly IPrintingHouseRepository _printingHouseRepository;
        private int? _printingHouseId;

        public int Id
        {
            set
            {
                try
                {
                    var printingHouse = _printingHouseRepository.ReadPrintingHouseById(value);
                    if (printingHouse == null)
                    {
                        throw new InvalidDataException(nameof(printingHouse));
                    }
                    textBoxAddress.Text = printingHouse.Address;
                    textBoxPhone.Text = printingHouse.PhoneNumber;
                    textBoxEmail.Text = printingHouse.Email;
                    textBoxName.Text = printingHouse.Name;
                    _printingHouseId = value;
                }
                catch (Exception ex)
                {
                    MessageBox.Show(ex.Message, "Ошибка при получении данных", MessageBoxButtons.OK, MessageBoxIcon.Error);
                    return;
                }
            }
        }
        public FormPrintingHouse(IPrintingHouseRepository printingHouseRepository)
        {
            InitializeComponent();
            _printingHouseRepository = printingHouseRepository ?? throw new ArgumentException(nameof(printingHouseRepository));
        }

        private void buttonSave_Click(object sender, EventArgs e)
        {
            try
            {
                if (string.IsNullOrWhiteSpace(textBoxPhone.Text) || string.IsNullOrWhiteSpace(textBoxEmail.Text) || string.IsNullOrWhiteSpace(textBoxName.Text))
                {
                    throw new Exception("Имеются незаполненные поля!");
                }

                if (_printingHouseId.HasValue)
                {
                    _printingHouseRepository.UpdatePrintingHouse(CreatePrintingHouse(_printingHouseId.Value));
                }
                else
                {
                    _printingHouseRepository.CreatePrintingHouse(CreatePrintingHouse(0));
                }
                Close();
            }
            catch (Exception ex)
            {
                MessageBox.Show(ex.Message, "Ошибка при сохранении", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
        }

        private void buttonCancel_Click(object sender, EventArgs e) => Close();

        private PrintingHouse CreatePrintingHouse(int id) => PrintingHouse.CreatePrintingHouse(id, textBoxName.Text, textBoxAddress.Text, textBoxPhone.Text, textBoxEmail.Text);
    }
}
