﻿using Dapper;
using Microsoft.Extensions.Logging;
using Newtonsoft.Json;
using Npgsql;
using ProjectPublishing.Entities;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace ProjectPublishing.Repositories.Implementations
{
    public class CustomerRepository : ICustomerRepository
    {
        private readonly IConnectionString _connectionString;
        private readonly ILogger<CustomerRepository> _logger;

        public CustomerRepository(IConnectionString connectionString, ILogger<CustomerRepository> logger)
        {
            _connectionString = connectionString;
            _logger = logger;
        }

        public void CreateCustomer(Customer customer)
        {
            _logger.LogInformation("Создание объекта");
            _logger.LogDebug("Объект: {json}", JsonConvert.SerializeObject(customer));
            try
            {
                using var connection = new NpgsqlConnection(_connectionString.ConnectionString);
                connection.Open();
                var queryInsert = @"
INSERT INTO Customer (PhoneNumber, Email, PublisherId)
VALUES (@PhoneNumber, @Email, @PublisherId)";
                connection.Execute(queryInsert, customer);
                connection.Close();
            }
            catch (Exception ex)
            {
                _logger.LogError(ex, "Ошибка при добавлении объекта");
                throw;
            }
        }

        public void UpdateCustomer(Customer customer)
        {
            _logger.LogInformation("Редактирование объекта");
            _logger.LogDebug("Объект: {json}", JsonConvert.SerializeObject(customer));
            try
            {
                using var connection = new NpgsqlConnection(_connectionString.ConnectionString);
                connection.Open();
                var queryUpdate = @"
UPDATE Customer
SET  
PhoneNumber=@PhoneNumber, 
Email=@Email,
PublisherId=@PublisherId
WHERE Id=@Id";
                connection.Execute(queryUpdate, customer);
                connection.Close();
            }
            catch (Exception ex)
            {
                _logger.LogError(ex, "Ошибка при редактировании объекта");
                throw;
            }
        }

        public void DeleteCustomer(int id)
        {
            _logger.LogInformation("Удаление объекта");
            _logger.LogDebug("Объект: {id}", id);
            try
            {
                using var connection = new NpgsqlConnection(_connectionString.ConnectionString);
                connection.Open();
                var queryDelete = @"
DELETE FROM Customer
WHERE Id=@id";
                connection.Execute(queryDelete, new { id });
                connection.Close();
            }
            catch (Exception ex)
            {
                _logger.LogError(ex, "Ошибка при удалении объекта");
                throw;
            }
        }

        public Customer ReadCustomerById(int id)
        {
            _logger.LogInformation("Получение объекта по ID");
            _logger.LogDebug("Объект: {id}", id);
            try
            {
                using var connection = new NpgsqlConnection(_connectionString.ConnectionString);
                connection.Open();
                var querySelect = @"
SELECT * FROM Customer
WHERE Id=@id";
                var customer = connection.QueryFirst<Customer>(querySelect, new { id });
                connection.Close();
                _logger.LogDebug("Найден объект: {json}", JsonConvert.SerializeObject(customer));
                return customer;
            }
            catch (Exception ex)
            {
                _logger.LogError(ex, "Ошибка при поиске объекта");
                throw;
            }
        }

        public IEnumerable<Customer> ReadCustomers()
        {
            _logger.LogInformation("Получение всех объектов");

            try
            {
                using var connection = new NpgsqlConnection(_connectionString.ConnectionString);
                var querySelect = "SELECT * FROM Customer";
                var contactPeople = connection.Query<Customer>(querySelect);
                _logger.LogDebug("Полученные объекты: {json}", JsonConvert.SerializeObject(contactPeople));
                return contactPeople;
            }
            catch (Exception ex)
            {
                _logger.LogError(ex, "Ошибка при чтении объектов");
                throw;
            }
        }

        
    }
}
