﻿using Dapper;
using Microsoft.Extensions.Logging;
using Newtonsoft.Json;
using Npgsql;
using ProjectPublishing.Entities;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace ProjectPublishing.Repositories.Implementations
{
    public class MaterialAdmissionRepository : IMaterialAdmissionRepository
    {
        private readonly IConnectionString _connectionString;
        private readonly ILogger<MaterialAdmissionRepository> _logger;
        public MaterialAdmissionRepository(IConnectionString connectionString, ILogger<MaterialAdmissionRepository> logger) 
        {
            _connectionString = connectionString;
            _logger = logger;
        }
        public void CreateMaterialsAdmission(MaterialsAdmission materialAdmission)
        {
            _logger.LogInformation("Создание объекта");
            _logger.LogDebug("Объект: {json}", JsonConvert.SerializeObject(materialAdmission));
            try
            {
                using var connection = new NpgsqlConnection(_connectionString.ConnectionString);
                connection.Open();
                var queryInsert = @"
INSERT INTO MaterialAdmission (Amount, MaterialId)
VALUES (@Amount, @MaterialId)";
                connection.Execute(queryInsert, materialAdmission);
                connection.Close();
            }
            catch (Exception ex)
            {
                _logger.LogError(ex, "Ошибка при добавлении объекта");
                throw;
            }
        }

        public IEnumerable<MaterialsAdmission> ReadMaterialsAdmission()
        {
            _logger.LogInformation("Получение всех объектов");

            try
            {
                using var connection = new NpgsqlConnection(_connectionString.ConnectionString);
                var querySelect = "SELECT * FROM MaterialAdmission";
                var materialAdmissions = connection.Query<MaterialsAdmission>(querySelect);
                _logger.LogDebug("Полученные объекты: {json}", JsonConvert.SerializeObject(materialAdmissions));
                return materialAdmissions;
            }
            catch (Exception ex)
            {
                _logger.LogError(ex, "Ошибка при чтении объектов");
                throw;
            }
        }

    }
}
